package com.ruoyi.yiqi.controller;

import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.annotation.RepeatSubmit;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.R;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.core.validate.AddGroup;
import com.ruoyi.common.core.validate.EditGroup;
import com.ruoyi.common.core.validate.QueryGroup;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.yiqi.entity.YqGHouse;
import com.ruoyi.yiqi.service.IYqGHouseService;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletResponse;
import javax.validation.constraints.NotEmpty;
import javax.validation.constraints.NotNull;
import java.util.Arrays;
import java.util.List;

/**
 * 房屋Controller
 *
 * @author ruoyi
 * @date 2021-11-15
 */
@Validated
@RequiredArgsConstructor(onConstructor_ = @Autowired)
@RestController
@RequestMapping("/yq/gas/house")
public class YqGHouseController extends BaseController {

    private final IYqGHouseService iGHouseService;

    /**
     * 查询房屋列表
     */
    @GetMapping("/list")
    public TableDataInfo<YqGHouse> list(@Validated(QueryGroup.class) YqGHouse bo) {
        return iGHouseService.queryPageList(bo);
    }

    /**
     * 导出房屋列表
     */
    @Log(title = "房屋", businessType = BusinessType.EXPORT)
    @GetMapping("/export")
    public void export(@Validated YqGHouse bo, HttpServletResponse response) {
        List<YqGHouse> list = iGHouseService.queryList(bo);
        ExcelUtil.exportExcel(list, "房屋", YqGHouse.class, response);
    }

    /**
     * 获取房屋详细信息
     */
    @GetMapping("/{id}")
    public R<YqGHouse> getInfo(@NotNull(message = "主键不能为空")
                                                  @PathVariable("id") Long id) {
        return R.ok(iGHouseService.getById(id));
    }

    /**
     * 新增房屋
     */
    @Log(title = "房屋", businessType = BusinessType.INSERT)
    @RepeatSubmit()
    @PostMapping()
    public R<Boolean> add(@Validated(AddGroup.class) @RequestBody YqGHouse bo) {
        return R.ok(iGHouseService.insertByBo(bo));
//        return toAjax(iGHouseService.insertByBo(bo) ? 1 : 0);
    }

    /**
     * 修改房屋
     */
    @Log(title = "房屋", businessType = BusinessType.UPDATE)
    @RepeatSubmit()
    @PutMapping()
    public R<Void> edit(@Validated(EditGroup.class) @RequestBody YqGHouse bo) {
        return toAjax(iGHouseService.updateByBo(bo) ? 1 : 0);
    }

    /**
     * 删除房屋
     */
    @Log(title = "房屋" , businessType = BusinessType.DELETE)
    @DeleteMapping("/{ids}")
    public R<Void> remove(@NotEmpty(message = "主键不能为空")
                                       @PathVariable Long[] ids) {
        return toAjax(iGHouseService.deleteWithValidByIds(Arrays.asList(ids), true) ? 1 : 0);
    }

    @GetMapping("/getList")
    public TableDataInfo<YqGHouse> getList(@Validated(QueryGroup.class) YqGHouse bo) {
        return iGHouseService.getList(bo);
    }

    @GetMapping("/getHouse")
    public R getHouse(Long unitId) {
        return R.ok(iGHouseService.getHouse(unitId));
    }
}
