package com.ruoyi.yiqi.service.impl;

import cn.hutool.core.bean.BeanUtil;
import com.baomidou.mybatisplus.core.conditions.Wrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.exception.ServiceException;
import com.ruoyi.common.helper.LoginHelper;
import com.ruoyi.common.utils.PageUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.framework.obs.FileUtil;
import com.ruoyi.framework.obs.ObsService;
import com.ruoyi.system.mapper.SysDictDataMapper;
import com.ruoyi.yiqi.entity.*;
import com.ruoyi.yiqi.enums.Constants;
import com.ruoyi.yiqi.mapper.YqGAreaMapper;
import com.ruoyi.yiqi.mapper.YqGBuildingMapper;
import com.ruoyi.yiqi.mapper.YqGHouseMapper;
import com.ruoyi.yiqi.mapper.YqGUnitMapper;
import com.ruoyi.yiqi.service.IYqGAreaService;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.ObjectUtils;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.math.BigDecimal;
import java.net.HttpURLConnection;
import java.net.URL;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.*;
import java.util.concurrent.atomic.AtomicInteger;
import java.util.function.Function;

/**
 * 小区Service业务层处理
 *
 * @author ruoyi
 * @date 2021-11-15
 */
@Service
@Slf4j
public class YqGAreaServiceImpl implements IYqGAreaService {


    private final static String path = "/zdsz/temporary/";

    @Resource
    private YqGAreaMapper gAreaMapper;
    @Resource
    private YqGBuildingMapper gBuildingMapper;
    @Resource
    private YqGUnitMapper gUnitMapper;
    @Resource
    private YqGHouseMapper gHouseMapper;
//    @Autowired
//    private YqGOrderMapper gOrderMapper;
//
//    @Autowired
//    private IGOrderPhotoService orderPhotoService;

    @Autowired
    private ObsService obsService;
    @Resource
    private SysDictDataMapper sysDictDataMapper;

    @Override
    public YqGArea queryById(Long id) {
        return gAreaMapper.selectVoById(id);
    }

    @Override
    public TableDataInfo<YqGArea> queryPageList(YqGArea bo) {
        Page<YqGArea> page = new Page<>(bo.getPageNum(), bo.getPageSize());
        Page<YqGArea> result = gAreaMapper.getList(bo,page);
        return PageUtils.buildDataInfo(result);
    }

    @Override
    public List<YqGArea> queryList(YqGArea bo) {
        return gAreaMapper.selectVoList(buildQueryWrapper(bo));
    }

    private LambdaQueryWrapper<YqGArea> buildQueryWrapper(YqGArea bo) {
        Map<String, Object> params = bo.getParams();
        LambdaQueryWrapper<YqGArea> lqw = Wrappers.lambdaQuery();
        lqw.like(StringUtils.isNotBlank(bo.getName()), YqGArea::getName, bo.getName());
        lqw.eq(bo.getUserId() != null, YqGArea::getUserId, bo.getUserId());
        return lqw;
    }

    @Override
    public Boolean insertByBo(YqGArea bo) {
        bo.setReleaseIs("001");
        YqGArea add = BeanUtil.toBean(bo, YqGArea.class);
        validEntityBeforeSave(add);
        boolean flag = false;
        Long count = gAreaMapper.checkDataList(bo);
        if(count == 0){
            flag = save(add);
            if (flag) {
                bo.setId(add.getId());
            }
        }
        return flag;
    }

    @Override
    public Boolean updateByBo(YqGArea bo) {
        YqGArea update = BeanUtil.toBean(bo, YqGArea.class);
        validEntityBeforeSave(update);
        return updateById(update);
    }

    /**
     * 保存前的数据校验
     *
     * @param entity 实体类数据
     */
    private void validEntityBeforeSave(YqGArea entity) {
        //TODO 做一些数据校验,如唯一约束
    }

    @Override
    public Boolean deleteWithValidByIds(Collection<Long> ids, Boolean isValid) {
        if (isValid) {
            //TODO 做一些业务上的校验,判断是否需要校验
        }
        return removeByIds(ids);
    }

    @Override
    public TableDataInfo<YqGArea> getList(YqGArea bo) {
        Page<YqGArea> page = new Page<>(bo.getPageNum(), bo.getPageSize());
        Page<YqGArea> list = gAreaMapper.getList(bo, page);
        DecimalFormat df = new DecimalFormat("0.00%");
        list.getRecords().forEach(i -> {
            //小区户数
            BigDecimal house = BigDecimal.valueOf(gAreaMapper.getHouse(i.getId()));
            //小区订单完成数
//            BigDecimal order = BigDecimal.valueOf(gOrderMapper.getCount(i.getId()));
//            if (house.compareTo(BigDecimal.ZERO) != 0 && order.compareTo(BigDecimal.ZERO) != 0) {
//                String rate = df.format(order.divide(house, 2, BigDecimal.ROUND_HALF_UP));
//                i.setRate(rate);
//            } else {
//                i.setRate("0.00%");
//            }
        });

        return PageUtils.buildDataInfo(list);
    }

    @Override
    public List<YqGArea> getArea() {
        return gAreaMapper.getArea();
    }

    @Override
    public List<GCensusVo> getExamineArea(Long areaId) {
        List<GCensusVo> list = gAreaMapper.getExamineArea(areaId);
        list.forEach(i -> {
            if (Constants.Dict.EXAMINE_STATUS001.getItemValue().equals(i.getName())) {
                i.setName(Constants.Dict.EXAMINE_STATUS001.getItemName());
            }
            if (Constants.Dict.EXAMINE_STATUS002.getItemValue().equals(i.getName())) {
                i.setName(Constants.Dict.EXAMINE_STATUS002.getItemName());
            }
        });
        return list;
    }

    @Override
    public List<GCensusVo> getExamineServe(Long id) {
        List<GCensusVo> list = gAreaMapper.getExamineServe(id);
        list.forEach(i -> {
            if (Constants.Dict.SERVICE_TYPE001.getItemValue().equals(i.getName())) {
                i.setName(Constants.Dict.SERVICE_TYPE001.getItemName());
            }
            if (Constants.Dict.SERVICE_TYPE002.getItemValue().equals(i.getName())) {
                i.setName(Constants.Dict.SERVICE_TYPE002.getItemName());
            }
            if (Constants.Dict.SERVICE_TYPE003.getItemValue().equals(i.getName())) {
                i.setName(Constants.Dict.SERVICE_TYPE003.getItemName());
            }
        });
        return list;
    }

    @Override
    public List<GExportVo> getExportList() {
        return gAreaMapper.getExportList();
    }


    @Override
    public List<GExportsVo> getExport(YqGArea bo) {
        return gAreaMapper.getExport(bo);
    }
    @Override
    public String getDictNameOfAffiliatedPackage(YqGArea bo){
        return gAreaMapper.getDictNameOfAffiliatedPackage(bo);
    }
    /**
     * 导入用户数据
     *
     * @param list            数据列表
     * @param isUpdateSupport 是否更新支持，如果已存在，则进行更新数据
     * @param operName        操作用户
     * @return 结果
     */
    @Transactional
    @Override
    public Map<String,Object> importUser(List<GImportVo> list, Boolean isUpdateSupport, String operName, int n, int successNum) {
        if (ObjectUtils.isEmpty(list) || list.size() == 0) {
            throw new ServiceException("导入数据不能为空！");
        }
        Map<String,Object> ret = new HashMap<>();
        StringBuilder successMsg = new StringBuilder();
        for (int i=n ;i< list.size() ;i++) {
            GImportVo vo = list.get(i);

            String districtValue = sysDictDataMapper.queryValueTextByKey("district",vo.getDistrict());
            if(StringUtils.isBlank(districtValue)){
                throw new ServiceException("" + "第" + (n+1) + "行行政区填写错误,导入失败!");
            }
//        }
//        for (GImportVo vo : list) {
            try {
                n++;
                // 验证是否存在这个小区
                if (StringUtils.isNotEmpty(vo.getAreaName())) {
                    YqGArea area = gAreaMapper.getByName(vo.getAreaName());
                    YqGBuilding building = new YqGBuilding();
                    YqGUnit unit = new YqGUnit();
                    YqGHouse house = new YqGHouse();
                    if (!ObjectUtils.isEmpty(area)) {
                        area = new YqGArea();
                        //小区zDsZ1a2b3c4d%1234!
                        area.setName(vo.getAreaName());
                        area.setCreateBy(operName);
                        area.setCreateTime(new Date());
                        area.setDistrict(districtValue);
                        gAreaMapper.insert(area);
                        if (!ObjectUtils.isEmpty(vo.getBuildName())) {
                            //楼宇
                            building.setName(vo.getBuildName());
                            building.setAreaId(area.getId());
                            building.setCreateBy(operName);
                            building.setCreateTime(new Date());
                            gBuildingMapper.insert(building);
                            if (!ObjectUtils.isEmpty(vo.getUnitName())) {
                                //单元
                                unit.setName(vo.getUnitName());
                                unit.setBuildingId(building.getId());
                                unit.setCreateBy(operName);
                                unit.setCreateTime(new Date());
                                gUnitMapper.insert(unit);
                                if (!ObjectUtils.isEmpty(vo.getHouseName())) {
                                    //房屋
                                    house.setName(vo.getHouseName());
                                    house.setUnitId(unit.getId());
                                    house.setCreateBy(operName);
                                    house.setCreateTime(new Date());
                                    gHouseMapper.insert(house);
                                    successNum++;
                                } else {
                                    throw new ServiceException("" + "第" + (n+1) + "行格式错误,导入失败!");
                                }
                            } else {
                                throw new ServiceException("" + "第" + (n+1) + "行格式错误,导入失败!");
                            }
                        } else {
                            throw new ServiceException("" + "第" + (n+1) + "行格式错误,导入失败!");
                        }
                    } else if (isUpdateSupport) {
//                        area.setUpdateBy(operName);
//                        area.setUpdateTime(new Date());
//                        gAreaMapper.updateById(area);
                        if (!ObjectUtils.isEmpty(vo.getBuildName())) {
                            building = gBuildingMapper.getByName(vo.getBuildName(), area.getId());
                            if (!ObjectUtils.isEmpty(building)) {
                                building = new YqGBuilding();
                                //楼宇
                                building.setName(vo.getBuildName());
                                building.setAreaId(area.getId());
                                building.setCreateBy(operName);
                                building.setCreateTime(new Date());
                                gBuildingMapper.insert(building);
                                if (!ObjectUtils.isEmpty(vo.getUnitName())) {
                                    //单元
                                    unit.setName(vo.getUnitName());
                                    unit.setBuildingId(building.getId());
                                    unit.setCreateBy(operName);
                                    unit.setCreateTime(new Date());
                                    gUnitMapper.insert(unit);
                                    if (!ObjectUtils.isEmpty(vo.getHouseName())) {
                                        //房屋
                                        house.setName(vo.getHouseName());
                                        house.setUnitId(unit.getId());
                                        house.setCreateBy(operName);
                                        house.setCreateTime(new Date());
                                        gHouseMapper.insert(house);
                                        successNum++;
                                    } else {
                                        throw new ServiceException("" + "第" + (n+1) + "行格式错误,导入失败!");
                                    }
                                } else {
                                    throw new ServiceException("" + "第" + (n+1) + "行格式错误,导入失败!");
                                }
                            } else {
//                                building.setUpdateBy(operName);
//                                building.setUpdateTime(new Date());
//                                gBuildingMapper.updateById(building);
                                if (!ObjectUtils.isEmpty(vo.getUnitName())) {
                                    unit = gUnitMapper.getByName(vo.getUnitName(), building.getId());
                                    if (!ObjectUtils.isEmpty(unit)) {
                                        unit = new YqGUnit();
                                        //单元
                                        unit.setName(vo.getUnitName());
                                        unit.setBuildingId(building.getId());
                                        unit.setCreateBy(operName);
                                        unit.setCreateTime(new Date());
                                        gUnitMapper.insert(unit);
                                        if (!ObjectUtils.isEmpty(vo.getHouseName())) {
                                            //房屋
                                            house.setName(vo.getHouseName());
                                            house.setUnitId(unit.getId());
                                            house.setCreateBy(operName);
                                            house.setCreateTime(new Date());
                                            gHouseMapper.insert(house);
                                            successNum++;
                                        }else{
                                            throw new ServiceException("" + "第" + (n+1) + "行格式错误,导入失败!");
                                        }
                                    } else {
//                                        unit.setUpdateBy(operName);
//                                        unit.setUpdateTime(new Date());
//                                        gUnitMapper.updateById(unit);
                                        if (!ObjectUtils.isEmpty(vo.getHouseName())) {
                                            house = gHouseMapper.getByName(vo.getHouseName(), unit.getId());
                                            if (!ObjectUtils.isEmpty(house)) {
                                                house = new YqGHouse();
                                                //房屋
                                                house.setName(vo.getHouseName());
                                                house.setUnitId(unit.getId());
                                                house.setCreateBy(operName);
                                                house.setCreateTime(new Date());
                                                gHouseMapper.insert(house);
                                                successNum++;
//                                            } else {
//                                                house.setUpdateBy(operName);
//                                                house.setUpdateTime(new Date());
//                                                gHouseMapper.updateById(house);
                                            }
                                        }else{
                                            throw new ServiceException("" + "第" + (n+1) + "行格式错误,导入失败!");
                                        }
                                    }
                                }else{
                                    throw new ServiceException("" + "第" + (n+1) + "行格式错误,导入失败!");
                                }
                            }
                        }else{
                            throw new ServiceException("" + "第" + (n+1) + "行格式错误,导入失败!");
                        }
                    }
                } else {
                    throw new ServiceException("" + "第" + (n+1) + "行格式错误,导入失败!");
                }
            } catch (Exception e) {
                throw new ServiceException("" + "第" + (n+1) + "行格式错误,导入失败!");
            }
            ret.put("bfb",String.format("%.2f",(Double.valueOf((n+1))/Double.valueOf(list.size()))*100));
            ret.put("n",n);
            ret.put("is_success",true);
            ret.put("successMsg","");
            ret.put("successNum",successNum);
            return ret;
        }
        successMsg.insert(0, "恭喜您，数据已全部导入成功！共 " + successNum + " 条");
        ret.put("bfb",String.format("%.2f",(Double.valueOf((n+1))/Double.valueOf(list.size()))*100));
        ret.put("n",n);
        ret.put("is_success",false);
        ret.put("successMsg",successMsg.toString());
        return ret;
    }

    /**
     * 导入用户数据
     *
     * @param list            数据列表
     * @param isUpdateSupport 是否更新支持，如果已存在，则进行更新数据
     * @param operName        操作用户
     * @return 结果
     */
    @Transactional
    @Override
    public String importApprovalExcel(List<GImportVo> list, Boolean isUpdateSupport, String operName) {
        if (!ObjectUtils.isEmpty(list) || list.size() == 0) {
            throw new ServiceException("导入数据不能为空！");
        }
        int successNum = 0;
        int n = 1;
        StringBuilder successMsg = new StringBuilder();
        for (GImportVo vo : list) {
            try {
                n++;
                // 验证是否存在这个小区
                if (!ObjectUtils.isEmpty(vo.getAreaName())) {
                    YqGArea area = gAreaMapper.getByName(vo.getAreaName());
                    YqGBuilding building = new YqGBuilding();
                    YqGUnit unit = new YqGUnit();
                    YqGHouse house = new YqGHouse();
                    if (!ObjectUtils.isEmpty(area)) {
                        building = gBuildingMapper.getByName(vo.getBuildName(), area.getId());
                        if (!ObjectUtils.isEmpty(building)) {
                            unit = gUnitMapper.getByName(vo.getUnitName(), building.getId());
                            if (!ObjectUtils.isEmpty(unit)) {
                                house = gHouseMapper.getByName(vo.getHouseName(), unit.getId());
                                if (!ObjectUtils.isEmpty(house)) {
                                    Long id = house.getId();
//                                    GOrder gOrder = gOrderMapper.getByHouseId(id);
//                                    if(!ObjectUtils.isEmpty(gOrder)){
//                                        gOrder.setServiceType("002");
//                                        gOrder.setOrderStatus("002");
//                                        gOrder.setExamineStatus("001");
//                                        String time = "2021-12-01 00:00:00";
//                                        SimpleDateFormat sf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
//                                        Date date = sf.parse(time);
//                                        gOrder.setTime(date);
//                                        gOrderMapper.updateById(gOrder);
//                                    }else{
//                                        GOrder order = new GOrder();
//                                        order.setOrderNum(String.valueOf(ProfileUtil.getAtomicCounter()));
//                                        order.setServiceType("002");
//                                        order.setHouseId(house.getId());
//                                        order.setOrderStatus("002");
//                                        order.setExamineStatus("001");
//                                        String time = "2021-12-01 00:00:00";
//                                        SimpleDateFormat sf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
//                                        Date date = sf.parse(time);
//                                        order.setTime(date);
//                                        gOrderMapper.insert(order);
//                                    }
                                    successNum++;
                                }else{
                                    throw new ServiceException("" + "第" + n + "行房间不存在,导入审批失败!");
                                }
                            }else{
                                throw new ServiceException("" + "第" + n + "行单元不存在,导入审批失败!");
                            }
                        }else{
                            throw new ServiceException("" + "第" + n + "行楼栋不存在,导入审批失败!");
                        }
                    }else{
                        throw new ServiceException("" + "第" + n + "行小区不存在,导入审批失败!");
                    }
                }else{
                    throw new ServiceException("" + "第" + n + "行小区名称未填写,导入审批失败!");
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        successMsg.insert(0, "恭喜您，数据已审批成功！共 " + successNum + " 条");
        return successMsg.toString();
    }


    @Transactional
    @Override
    public String importUserA(List<GImportVo> list, Boolean isUpdateSupport, String operName) {
        if (!ObjectUtils.isEmpty(list) || list.size() == 0) {
            throw new ServiceException("导入数据不能为空！");
        }
        int successNum = 0;
        int n = 1;
        StringBuilder successMsg = new StringBuilder();
        for (GImportVo vo : list) {
            try {
                n++;
                // 验证是否存在这个小区
                if (!ObjectUtils.isEmpty(vo.getAreaName())) {
                    YqGArea area = gAreaMapper.getByName(vo.getAreaName());
                    YqGBuilding building = new YqGBuilding();
                    YqGUnit unit = new YqGUnit();
                    YqGHouse house = new YqGHouse();
//                    GOrder order = new GOrder();
                    if (!ObjectUtils.isEmpty(area)) {
                        area = new YqGArea();
                        //小区
                        area.setName(vo.getAreaName());
                        area.setCreateBy(operName);
                        area.setCreateTime(new Date());
                        gAreaMapper.insert(area);
                        if (!ObjectUtils.isEmpty(vo.getBuildName())) {
                            //楼宇
                            building.setName(vo.getBuildName());
                            building.setAreaId(area.getId());
                            building.setCreateBy(operName);
                            building.setCreateTime(new Date());
                            gBuildingMapper.insert(building);
                            if (!ObjectUtils.isEmpty(vo.getUnitName())) {
                                //单元
                                unit.setName(vo.getUnitName());
                                unit.setBuildingId(building.getId());
                                unit.setCreateBy(operName);
                                unit.setCreateTime(new Date());
                                gUnitMapper.insert(unit);
                                if (!ObjectUtils.isEmpty(vo.getHouseName())) {
                                    //房屋
                                    house.setName(vo.getHouseName());
                                    house.setUnitId(unit.getId());
                                    house.setCreateBy(operName);
                                    house.setCreateTime(new Date());
                                    gHouseMapper.insert(house);
                                    successNum++;
//                                    order.setOrderNum(String.valueOf(ProfileUtil.getAtomicCounter()));
//                                    order.setServiceType("002");
//                                    order.setHouseId(house.getId());
//                                    order.setOrderStatus("002");
//                                    order.setExamineStatus("001");
                                    String time = "2021-12-01 00:00:00";
                                    SimpleDateFormat sf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                                    Date date = sf.parse(time);
//                                    order.setTime(date);
//                                    gOrderMapper.insert(order);

                                } else {
                                    throw new ServiceException("" + "第" + n + "行格式错误,导入失败!");
                                }
                            } else {
                                throw new ServiceException("" + "第" + n + "行格式错误,导入失败!");
                            }
                        } else {
                            throw new ServiceException("" + "第" + n + "行格式错误,导入失败!");
                        }
                    } else if (isUpdateSupport) {
                        area.setUpdateBy(operName);
                        area.setUpdateTime(new Date());
                        gAreaMapper.updateById(area);
                        if (!ObjectUtils.isEmpty(vo.getBuildName())) {
                            building = gBuildingMapper.getByName(vo.getBuildName(), area.getId());
                            if (!ObjectUtils.isEmpty(building)) {
                                building = new YqGBuilding();
                                //楼宇
                                building.setName(vo.getBuildName());
                                building.setAreaId(area.getId());
                                building.setCreateBy(operName);
                                building.setCreateTime(new Date());
                                gBuildingMapper.insert(building);
                                if (!ObjectUtils.isEmpty(vo.getUnitName())) {
                                    //单元
                                    unit.setName(vo.getUnitName());
                                    unit.setBuildingId(building.getId());
                                    unit.setCreateBy(operName);
                                    unit.setCreateTime(new Date());
                                    gUnitMapper.insert(unit);
                                    if (!ObjectUtils.isEmpty(vo.getHouseName())) {
                                        //房屋
                                        house.setName(vo.getHouseName());
                                        house.setUnitId(unit.getId());
                                        house.setCreateBy(operName);
                                        house.setCreateTime(new Date());
                                        gHouseMapper.insert(house);
                                        successNum++;
//                                        order.setOrderNum(String.valueOf(ProfileUtil.getAtomicCounter()));
//                                        order.setServiceType("002");
//                                        order.setHouseId(house.getId());
//                                        order.setOrderStatus("002");
//                                        order.setExamineStatus("001");
                                        String time = "2021-12-01 00:00:00";
                                        SimpleDateFormat sf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                                        Date date = sf.parse(time);
//                                        order.setTime(date);
//                                        gOrderMapper.insert(order);
                                    } else {
                                        throw new ServiceException("" + "第" + n + "行格式错误,导入失败!");
                                    }
                                } else {
                                    throw new ServiceException("" + "第" + n + "行格式错误,导入失败!");
                                }
                            } else {
                                building.setUpdateBy(operName);
                                building.setUpdateTime(new Date());
                                gBuildingMapper.updateById(building);
                                if (!ObjectUtils.isEmpty(vo.getUnitName())) {
                                    unit = gUnitMapper.getByName(vo.getUnitName(), building.getId());
                                    if (!ObjectUtils.isEmpty(unit)) {
                                        unit = new YqGUnit();
                                        //单元
                                        unit.setName(vo.getUnitName());
                                        unit.setBuildingId(building.getId());
                                        unit.setCreateBy(operName);
                                        unit.setCreateTime(new Date());
                                        gUnitMapper.insert(unit);
                                        if (!ObjectUtils.isEmpty(vo.getHouseName())) {
                                            //房屋
                                            house.setName(vo.getHouseName());
                                            house.setUnitId(unit.getId());
                                            house.setCreateBy(operName);
                                            house.setCreateTime(new Date());
                                            gHouseMapper.insert(house);
                                            successNum++;
//                                            order.setOrderNum(String.valueOf(ProfileUtil.getAtomicCounter()));
//                                            order.setServiceType("002");
//                                            order.setHouseId(house.getId());
//                                            order.setOrderStatus("002");
//                                            order.setExamineStatus("001");
                                            String time = "2021-12-01 00:00:00";
                                            SimpleDateFormat sf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                                            Date date = sf.parse(time);
//                                            order.setTime(date);
//                                            gOrderMapper.insert(order);
                                        }else{
                                            throw new ServiceException("" + "第" + n + "行格式错误,导入失败!");
                                        }
                                    } else {
                                        unit.setUpdateBy(operName);
                                        unit.setUpdateTime(new Date());
                                        gUnitMapper.updateById(unit);
                                        if (!ObjectUtils.isEmpty(vo.getHouseName())) {
                                            house = gHouseMapper.getByName(vo.getHouseName(), unit.getId());
                                            if (!ObjectUtils.isEmpty(house)) {
                                                house = new YqGHouse();
                                                //房屋
                                                house.setName(vo.getHouseName());
                                                house.setUnitId(unit.getId());
                                                house.setCreateBy(operName);
                                                house.setCreateTime(new Date());
                                                gHouseMapper.insert(house);
                                                successNum++;
//                                                order.setOrderNum(String.valueOf(ProfileUtil.getAtomicCounter()));
//                                                order.setServiceType("002");
//                                                order.setHouseId(house.getId());
//                                                order.setOrderStatus("002");
//                                                order.setExamineStatus("001");
                                                String time = "2021-12-01 00:00:00";
                                                SimpleDateFormat sf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
                                                Date date = sf.parse(time);
//                                                order.setTime(date);
//                                                gOrderMapper.insert(order);
                                            } else {
                                                house.setUpdateBy(operName);
                                                house.setUpdateTime(new Date());
                                                gHouseMapper.updateById(house);
//                                                order = gOrderMapper.getByHouseId(house.getId());
//                                                if (!ObjectUtils.isEmpty(order)){
//                                                    order = new GOrder();
//                                                    order.setOrderNum(String.valueOf(ProfileUtil.getAtomicCounter()));
//                                                    order.setServiceType("002");
//                                                    order.setHouseId(house.getId());
//                                                    order.setOrderStatus("002");
//                                                    order.setExamineStatus("001");
//                                                    String time = "2021-12-01 00:00:00";
//                                                    SimpleDateFormat sf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
//                                                    Date date = sf.parse(time);
//                                                    order.setTime(date);
//                                                    gOrderMapper.insert(order);
//                                                }
                                            }
                                        }else{
                                            throw new ServiceException("" + "第" + n + "行格式错误,导入失败!");
                                        }
                                    }
                                }else{
                                    throw new ServiceException("" + "第" + n + "行格式错误,导入失败!");
                                }
                            }
                        }else{
                            throw new ServiceException("" + "第" + n + "行格式错误,导入失败!");
                        }
                    } else {
                        throw new ServiceException("" + "第" + n + "行小区已存在,导入失败!");
                    }
                } else {
                    throw new ServiceException("" + "第" + n + "行格式错误,导入失败!");
                }
            } catch (Exception e) {
                throw new ServiceException("" + "第" + n + "行格式错误,导入失败!");
            }
        }
        successMsg.insert(0, "恭喜您，数据已全部导入成功！共 " + successNum + " 条");
        return successMsg.toString();
    }

    @Async
    @Override
    @Transactional(rollbackFor = Exception.class)
    public void write(List<Long> ids) {
//        List<GDownloadVo> list = orderPhotoService.getAll(ids);
//        list.forEach(x -> {
//            if (!ObjectUtils.isEmpty(x.getPicUrl())) {
//                // 创建最外层文件夹
//                File file = new File(path);
//                if (!file.exists()) {
//                    file.mkdirs();
//                }
//                // 创建小区
//                File headFile = new File(file.getPath() + "/" + x.getArea());
//                if (!headFile.exists()) {
//                    headFile.mkdirs();
//                }
//                // 小区下再次创建小区，本级用于导出
//                File areaFile = new File(headFile.getPath() + "/" + x.getArea());
//                if (!areaFile.exists()) {
//                    areaFile.mkdirs();
//                }
//                // 楼
//                File buildingFile = new File(areaFile.getPath() + "/" + x.getBuilding());
//                if (!buildingFile.exists()) {
//                    buildingFile.mkdirs();
//                }
//                // 单元
//                File unitFile = new File(buildingFile.getPath() + "/" + x.getUnit());
//                if (!unitFile.exists()) {
//                    unitFile.mkdirs();
//                }
//                // 房间
//                File houseFile = new File(unitFile.getPath() + "/" + x.getHouse());
//                if (!houseFile.exists()) {
//                    houseFile.mkdirs();
//                }
//                try {
//                    obsService.download(houseFile.getPath(), x.getPicUrl().substring(x.getPicUrl().lastIndexOf("/") + 1));
//                    System.out.println("22222222222222222222222222");
//                } catch (IOException e) {
//                    e.printStackTrace();
//                }
//            }
//        });
    }

    @Override
    public void upload(HttpServletRequest request, HttpServletResponse response, Long id) {
        YqGArea po = gAreaMapper.selectById(id);
        // 设置文件夹时间戳和路径位置
        SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
        String format = sdf.format(new Date());
        // 文件夹的路径，写入服务器的 /tmp 文件夹下
        String filePath = path + po.getName();
//        String filePath = "D:\\zhsq_qk\\御景名都B";
        // 导出的zip路径
        String zipFilePath = path + "tmp_" + format + ".zip";
        // 导出的zip名字
        String zipName = "PHOTO_" + format + ".zip";

        // 调用工具类压缩文件夹
        FileOutputStream fileOutputStream = null;
        try {
            fileOutputStream = new FileOutputStream(new File(zipFilePath));
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
        FileUtil.toZip(filePath, fileOutputStream, true);

        // 调用工具类设置响应格式
        try {
            FileUtil.downLoadFile(request, response, zipName, zipFilePath);
        } catch (UnsupportedEncodingException e) {
            log.warn("下载小区压缩文件service："+e);
            log.error("用户：{}导出SQL压缩包失败{}" + e);
            throw new RuntimeException("导出失败~");
        }
//        deleteDirectory(filePath);
    }

    @Override
    public Boolean releaseIs(Long id) {
        YqGArea bo = new YqGArea();
        bo.setId(id);
        bo.setReleaseBy(LoginHelper.getLoginUser().getUsername());
        bo.setReleaseIs("002");
        YqGArea update = BeanUtil.toBean(bo, YqGArea.class);
        validEntityBeforeSave(update);
        return updateById(update);
    }

    @Override
    public List<String> getPhoto(Long id) {
        return gAreaMapper.getPhoto(id);
    }

    @Override
    public Boolean getForName(String name, String id) {
        YqGArea vo = gAreaMapper.getForName(name, id);
        if (vo == null) {
            return true;
        } else {
            return false;
        }
    }

    @Override
    public List<GAreaExportVo> getExportHouse(Long areaId) {
        //全部房屋
        List<GAreaExportVo> allHouse = gAreaMapper.getAllHouse(areaId);
        //已完工
        List<GAreaExportVo> orderHouse = gAreaMapper.getOrderHouse(areaId);
        allHouse.stream().forEach(i->{
            if (orderHouse.stream().filter(x -> x.getId().equals(i.getId())).findFirst().orElse(null) != null) {
                i.setStatus("已完工");
            }else{
                i.setStatus("未完工");
            }
        });
        return allHouse;
    }

    @Override
    public List<GCensusVo> getAreaSum(Long areaId) {
        List<GCensusVo> list = gAreaMapper.getConform(areaId);
        AtomicInteger aa = new AtomicInteger(gAreaMapper.getHouseCount(areaId));
        list.forEach(i -> {
            if (Constants.Dict.SERVICE_TYPE002.getItemValue().equals(i.getName())) {
                i.setName(Constants.Dict.SERVICE_TYPE002.getItemName());
            }
            if (Constants.Dict.SERVICE_TYPE003.getItemValue().equals(i.getName())) {
                i.setName(Constants.Dict.SERVICE_TYPE003.getItemName());
            }
//            if (Constants.Dict.SERVICE_TYPE004.getItemValue().equals(i.getName())) {
//                i.setName(Constants.Dict.SERVICE_TYPE004.getItemName());
//            }
//            if (Constants.Dict.SERVICE_TYPE005.getItemValue().equals(i.getName())) {
//                i.setName(Constants.Dict.SERVICE_TYPE005.getItemName());
//            }
//            if (Constants.Dict.SERVICE_TYPE006.getItemValue().equals(i.getName())) {
//                i.setName(Constants.Dict.SERVICE_TYPE006.getItemName());
//            }
            aa.set(aa.get() - i.getValue());
        });
        GCensusVo vo = new GCensusVo();
        vo.setName("未施工");
        vo.setValue(aa.get());
        list.add(vo);
        return list;
    }

    /**
     * 将图片写入到磁盘
     *
     * @param img      图片数据流
     * @param fileName 文件保存时的名称
     */

    public void writeImageToDisk(String path, byte[] img, String fileName) {
        try {
            File file = new File(path + fileName);

            FileOutputStream fops = new FileOutputStream(file);

            fops.write(img);

            fops.flush();

            fops.close();

            log.info("图片已经写入到C盘");

        } catch (Exception e) {
            e.printStackTrace();

        }

    }

    /**
     * 从输入流中获取数据
     *
     * @param inStream 输入流
     * @return
     * @throws Exception
     */

    public static byte[] readInputStream(InputStream inStream) throws Exception {
        ByteArrayOutputStream outStream = new ByteArrayOutputStream();
        byte[] buffer = new byte[1024];
        int len = 0;
        while ((len = inStream.read(buffer)) != -1) {
            outStream.write(buffer, 0, len);

        }
        inStream.close();
        return outStream.toByteArray();
    }
    private void deleteFile(File file) {
        if (file.exists()) {
            if (file.isFile()) {
                file.delete();
            } else if (file.isDirectory()) {
                File[] files = file.listFiles();
                for (File f : files) {
                    this.deleteFile(f);
                }
            }
            file.delete();
        } else {
            System.out.println("所删除的" + file.getAbsolutePath() + "不存在！");
        }
    }
    /**
     * 删除目录及目录下的文件
     *
     * @param dir：要删除的目录的文件路径
     * @return 目录删除成功返回true，否则返回false
     */
    @Async
    @Override
    public void deleteDirectory(String dir) {
        deleteFile(new File(path + dir));
    }

    @Override
    public boolean saveBatch(Collection<YqGArea> entityList, int batchSize) {
        return false;
    }

    @Override
    public boolean saveOrUpdateBatch(Collection<YqGArea> entityList, int batchSize) {
        return false;
    }

    @Override
    public boolean updateBatchById(Collection<YqGArea> entityList, int batchSize) {
        return false;
    }

    @Override
    public boolean saveOrUpdate(YqGArea entity) {
        return false;
    }

    @Override
    public YqGArea getOne(Wrapper<YqGArea> queryWrapper, boolean throwEx) {
        return null;
    }

    @Override
    public Optional<YqGArea> getOneOpt(Wrapper<YqGArea> queryWrapper, boolean throwEx) {
        return Optional.empty();
    }

    @Override
    public Map<String, Object> getMap(Wrapper<YqGArea> queryWrapper) {
        return new HashMap<>();
    }

    @Override
    public <V> V getObj(Wrapper<YqGArea> queryWrapper, Function<? super Object, V> mapper) {
        return null;
    }

    @Override
    public BaseMapper<YqGArea> getBaseMapper() {
        return null;
    }

    @Override
    public Class<YqGArea> getEntityClass() {
        return null;
    }
}
